package criteres;

import don.DonneeID;

/** Critères à appliquer sur des données
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public abstract class Critere
{
	/** chaine erreur */
	public final static String CHAINE_ERREUR ="erreur (critere)";
	
	/** Valeur rendue si la donnée ne peut pas être évaluée par le critère */
	public final static int CODE_ERROR=Integer.MAX_VALUE;

	/** Valeur rendue si la donnée correspond au critère, mais n'a pas de valeur particulière */
	public final static int CODE_OK=0;

	/** @return le nombre de séries qui peuvent être construites selon le critère */
	public abstract int getNbSeries();

	/** Evalue un objet selon le critère
	 * cette fonction doit balayer les nombres de 0 à n
	 * @param obj
	 * @return le résultat de l'évaluation, CODE_ERROR si le critère n'est pas applicable
	 */
	public abstract Info evaluer(DonneeID obj);

	/** Construit le test qui est la négation d'un critère :
	 * si X.evaluer(obj) est OK, alors X.not().evaluer(obj) rend une erreur
	 * sinon X.not().evaluer(obj) rend OK
	 * @return la négation du critère 
	 */
	public Test not()
	{
		final Critere pere=this;
		return new Test()
		{
			public boolean correspond(DonneeID obj)
			{return pere.evaluer(obj).isError();}

			public String semantique()
			{return "";}

			public boolean adjectif()
			{return pere.adjectif();}

			@Override
			public int compare(Info i0, Info i1) {
				return -pere.compare(i0, i1);
			}
		};
	}
	
	/**
	 * @return le critere
	 */
	public Test test(){
		final Critere pere=this;
		return new Test()
		{
			public boolean correspond(DonneeID obj)
			{return !pere.evaluer(obj).isError();}

			public String semantique()
			{return "";}

			public boolean adjectif()
			{return pere.adjectif();}

			@Override
			public int compare(Info i0, Info i1) {
				return pere.compare(i0, i1);
			}
		};
	}

	/** Rend la définition du critère
	 * @return ce que filtre le critère
	 */
	public abstract String semantique();

	/** Rend la définition du critère
	 * @param i
	 * @return donne la sémantique d'une évaluation par la critère
	 */
	public String semantique(Paire i)
	{return CHAINE_ERREUR+" de base";}

	/** Rend la définition du critère
	 * @param i
	 * @return donne la sémantique d'une évaluation par la critère
	 */
	public String semantique(Info i)
	{
		if(i instanceof Paire)return semantique((Paire)i);
		else return semantique((Singleton)i);
	}

	/** Rend la définition du critère
	 * @param i
	 * @return donne la sémantique d'une évaluation par la critére
	 */
	public abstract String semantique(Singleton i);

	/** Rend la définition du critére
	 * @param obj
	 * @return qualifie l'objet (par un texte)
	 */
	public String qualifier(DonneeID obj)
	{
		Info res =evaluer(obj);
		if(res  instanceof Paire) return semantique((Paire)res);
		else return semantique((Singleton)res);
	}
	
	/** méthode qui dit si c'est un adjectif
	 * @return un booléen qui dit si le critère correspond à un adjectif
	 */
	public abstract boolean adjectif();
	
	/* (non-Javadoc)
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	protected int compare(Paire i0, Paire i1){return 0;};
	
	protected int compare(Singleton i0, Singleton i1){
		return i0.get()-i1.get();
	}

	/**
	 * @param g0
	 * @param g1
	 * @return @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(Info g0, Info g1) {
		if(g0 instanceof Paire && g1 instanceof Paire){
			return compare((Paire)g0,(Paire)g1);
		}else if(g0 instanceof Singleton && g1 instanceof Singleton){
			return compare((Singleton)g0,(Singleton)g1);
		}else return 0;
	}
	
	
}
