package don;

import java.io.Serializable;
import java.util.HashMap;

import org.jfree.chart.annotations.XYIDAnnotation;
import org.jfree.ui.TextAnchor;

import exceptions.ExceptionIntegrite;

/**
 * classe donnée ID (indidu, variable, ...)
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public abstract class DonneeID implements Serializable, Verifiable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** l'annotation de la donnée */
	private String m_annotation;

	/** le groupe de variables auquel appartient la donnée */
	private Groupe m_gr;

	/** les coordonnées de la donnée sur les axes */
	private double[] m_coord;

	/** les coordonnées en X des annotations sur les axes 
	 * m_coord_annotX[i][j] est la coordonnée en X si l'axe i est en X et l'axe j en Y
	 */
	private HashMap<Continue,HashMap <Continue,Double>>  m_coord_annotX;

	/** les coordonnées en Y des annotations sur les axes 
	 * m_coord_annotY[i][j] est la coordonnée en Y si l'axe i est en Y et l'axe j en Y
	 */
	private HashMap<Continue,HashMap <Continue,Double>> m_coord_annotY;

	/** les cosé de la donnée sur les axes */
	private double[] m_cos2;

	/** les contributions de la donnée aux axes */
	private double[] m_contrib;

	/** le statut de la donnée (active/illustrative) */
	private boolean m_active;

	private TextAnchor m_anchor;

	/**
	 * Constructeur
	 * 
	 * @param annotation
	 * @param active
	 * @param gr
	 * @param coord
	 * @param cos2
	 * @param contrib
	 */
	public DonneeID(String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib) {
		m_annotation = annotation;
		m_active = active;
		m_gr = gr;
		m_coord = coord;
		m_cos2 = cos2;
		m_contrib = contrib;
		//valeur par défaut pour l'annotation
		if(m_coord!=null)calcCoordAnnotDefaut();
	}

	/**
	 * @return le statut (active/illustrative) de la donnée
	 */
	public boolean isActive() {
		return m_active;
	}

	/**
	 * Fixe le statut (active/illustrative) de la donnée
	 * @param active
	 */
	public void setActive(boolean active) {
		m_active = active;
	}

	/**
	 * @return l'annotation de la donnée
	 */
	public String getAnnotation() {
		return m_annotation;
	}

	/**
	 * Fixe l'annotation de la donnée
	 * @param annotation
	 */
	public void setAnnotation(String annotation) {
		m_annotation = annotation;
	}

	/**
	 * @param num le numéro de l'axe
	 * @return les contributions de la donnée aux axes factoriels
	 */
	public double getContrib(int num) {
		return m_contrib[num];
	}

	/**
	 * Fixe les contributions de la donnée aux axes factoriels
	 * @param contrib
	 */
	public void setContrib(double[] contrib) {
		m_contrib = contrib;
		calcCoordAnnotDefaut();
	}

	/**
	 * @param num le numéro de l'axe 
	 * @return les coordonnées (sur les axes factoriels) de la donnée
	 */
	public double getCoord(int num) {
		return m_coord[num];
	}
	
	/**
	 * @param c l'axe ou la continue concerné
	 * @return les coordonnées (sur les axes factoriels) de la donnée
	 */
	public double getCoord(Continue c) {
		return c.getValeur(this);
	}
	
	

	/**
	 * Fixe les coordonnées (sur les axes factoriels) de la donnée
	 * 
	 * @param coord
	 */
	public void setCoord(double[] coord)
	{
		m_coord = coord;
		calcCoordAnnotDefaut();
	}

	private void calcCoordAnnotDefaut() {
		m_coord_annotX=new HashMap<Continue, HashMap<Continue,Double>>();
		m_coord_annotY=new HashMap<Continue, HashMap<Continue,Double>>();
	}

	/**
	 * @param num le numéro de l'axe
	 * @return les cos2 de la donnée
	 */
	public double getCos2(int num) {
		if(m_cos2==null)return 0.0;
		else return m_cos2[num];
	}

	/**
	 * Fixe les cos2 de la donnée
	 * 
	 * @param cos2
	 */
	public void setCos2(double[] cos2) {
		m_cos2 = cos2;
		calcCoordAnnotDefaut();
	}

	/**
	 * 
	 * @return le groupe de la donnée
	 */
	public Groupe getGr() {
		return m_gr;
	}

	/**
	 * @param axeX le numéro de l'axe en X 
	 * @param axeY le numéro de l'axe en Y
	 * @return la coordonnée de l'annotation en X
	 */
	public double getXannot(Continue axeX, Continue axeY){
		Double xannot = null;
		HashMap<Continue, Double> hashMap = m_coord_annotX.get(axeX);
		if(hashMap!=null)
			xannot = hashMap.get(axeY);
		if(xannot==null){
			xannot=axeX.getValeur(this);
			setXannot(axeX, axeY, xannot);
		}
		return xannot;
	}

	/**
	 * @param axeX le numéro de l'axe en X 
	 * @param axeY le numéro de l'axe en Y
	 * @return la coordonnée de l'annotation en Y
	 */
	public double getYannot(Continue axeX, Continue axeY){
		Double yannot = null;
		HashMap<Continue, Double> hashMap = m_coord_annotY.get(axeX);
		if(hashMap!=null)
			yannot = hashMap.get(axeY);
		if(yannot==null){
			yannot=axeY.getValeur(this);
			setYannot(axeX, axeY, yannot);
		}
		return yannot;
	}

	/**
	 * @return l'ancrage de l'annotation
	 */
	public TextAnchor getAnchor(){
		if(m_anchor==null)return XYIDAnnotation.DEFAULT_TEXT_ANCHOR;
		else return m_anchor;}

	/**
	 * Modifie les coordonnée de l'annotation
	 * @param axeX le numéro de l'axe en X 
	 * @param axeY le numéro de l'axe en Y
	 * @param xannot
	 * @param yannot 
	 * @param anchor 
	 */
	public void setXYannot(Continue axeX, Continue axeY, double xannot,double yannot,TextAnchor anchor)
	{
		setXannot(axeX, axeY, xannot);		
		setYannot(axeX, axeY, yannot);		
		m_anchor=anchor;		
	}

	private void setYannot(Continue axeX, Continue axeY, double yannot) {
		HashMap<Continue, Double> tab;
		tab = m_coord_annotY.get(axeX);
		if(tab==null){
			tab=new HashMap<Continue, Double>();
			m_coord_annotY.put(axeX, tab);
		}
		tab.put(axeY, yannot);
	}

	private void setXannot(Continue axeX, Continue axeY, double xannot) {
		HashMap<Continue, Double> tab = m_coord_annotX.get(axeX);
		if(tab==null){
			tab=new HashMap<Continue, Double>();
			m_coord_annotX.put(axeX, tab);
		}
		tab.put(axeY, xannot);
	}




	/**
	 * met é jour le groupe de la donnée avertit si nécessaire l'ancien et
	 * le nouveau groupe
	 * 
	 * @param gr
	 */
	public void setGr(Groupe gr) {
		if (m_gr != null) {
			// TODO avertir le groupe que la donnée ne lui appartient plus
		}
		if (!gr.getDonneesID().contains(this)) {
			// si le nouveau groupe n'est pas encore au courant que la
			// donnée lui appartient, l'avertir
			gr.addDonneesID(this);
		}
		m_gr = gr;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String chaine = this.getClass().getSimpleName()+" "+m_annotation;
		if(m_active)chaine+=" actif/ve";
		else chaine+=" illustratif/ve";
		if (m_gr != null)   chaine += " (" + m_gr.getNum() + ")";
		else   chaine += " (pas de groupe)";
		return chaine;
	}



	public void verifierIntegrite() throws ExceptionIntegrite {
		String type="("+this.getClass().getSimpleName()+")";
		if(m_annotation==null)throw new ExceptionIntegrite("donnée "+type+" sans annotation");
		if(!(this instanceof Groupe)) {
			if(m_gr==null)throw new ExceptionIntegrite("donnée "+type+" sans groupe : "+m_annotation);
			else if(m_gr.getDonneesID()!=null) { 
				if(!m_gr.getDonneesID().contains(this)) {
					m_gr.addDonneesID(this);					
				}
			}else {
				m_gr.addDonneesID(this);
			}
		}
	}

}

