package don;

import ihm.FenetreErreur;

import java.io.Serializable;
import java.security.InvalidParameterException;
import java.util.Vector;

import exceptions.ExceptionIntegrite;

/**
 * classe qui stocke le jeu de donnees
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public class DonneeParse implements Serializable, Verifiable
{
	private static final long serialVersionUID = -3082877286069225411L;

	/** nom des colonnes */
	private String m_colName[];

	/** les axes */
	private Vector<Dimension> m_axes;

	/** liste des variables nominales */
	private Vector<Nominale> m_nominales;

	/** liste des individus moyens */
	private Vector<IndividuMoyen> m_individusMoyen;
	
//	/** liste des individus partiels s'ils existent */
//	private Vector<IndividuPartiel> m_individusPartiels;

	/** listes des variables moyennes */
	private Vector<ContinueMoyenne> m_continuesMoyennes;

	/** listes des axes partiels */
	private Vector<AxePartiel> m_axesPartiels;

	/** liste des groupes de variables */
	private Vector<Groupe> m_groupes;

	/** pourcentages d'inertie des axes factoriels   */
	private double[] m_inerties;

	/**
	 * Les différents types d'analyse supportés + "AUTRE" pour un type non spécifié
	 */
	public enum TypeAnalyse{
		/** PCA ou ACP : Analyse des Composantes Principales */
		PCA, 
		/** MCA ou ACM : Analyse des Correspondances Multiples */
		MCA, 
		/** MFA ou AFM : Analyse Factorielle Multiple */
		MFA, 
		/**	CA ou AC : Analyse des Correspondances */
		CA, 
		/** AFDM : Analyse Factorielle de Données Mixtes */
		AFDM,
		/** DATAFRAME : manipulation d'un simple dataframe */
		DATAFRAME,
		/** type d'analyse non spécifié */
		AUTRE
	}
	private TypeAnalyse m_type;


	/** Constructeur
	 * @param nomsColonnes : les noms des différentes colonnes du tableau équivalent
	 * @param axe : le nombre d'axe
	 * @param type  : le type de l'analyse
	 */
	public DonneeParse(String[] nomsColonnes, int axe, TypeAnalyse type)
	{
		m_colName = nomsColonnes;
		setNbAxe(axe);
		m_type=type;
		m_nominales = new Vector<Nominale>();
		m_individusMoyen = new Vector<IndividuMoyen>();
//		m_individusPartiels =  new Vector<IndividuPartiel>();
		m_continuesMoyennes = new Vector<ContinueMoyenne>();
		m_axesPartiels = new Vector<AxePartiel>();
		m_groupes = new Vector<Groupe>();
		m_inerties=new double[axe];
	}

	/** Constructeur : le type d'analyse est mis à AUTRE par défaut
	 * @param nomsColonnes : les noms des différentes colonnes du tableau équivalent
	 * @param axe : le nombre d'axe
	 */
	public DonneeParse(String[] nomsColonnes, int axe){
		this(nomsColonnes,axe,TypeAnalyse.AUTRE);
	}
	/**
	 * @return the m_colName
	 */
	public String[] getColName() {
		return m_colName;
	}

	/**
	 * @param colName 
	 *            the m_colName to set
	 */
	public void setColName(String[] colName) {
		m_colName = colName;
	}

	/**
	 * @return l'indice où commencent les noms de variables dans le tableau des noms de colonnes
	 */
	public int getDebutNomsVariables() {
		int deb = getNbAxe() * 3;
		deb = deb + 4;
		return deb;
	}

	/**
	 * @return the m_nbAxe
	 */
	public int getNbAxe() {
		return m_axes.size();
	}

	/**
	 * @param nbAxe
	 *            the m_nbAxe to set
	 */
	public void setNbAxe(int nbAxe) {
		m_axes = new Vector<Dimension>();
		for (int i = 1; i <= nbAxe; i++) {
			m_axes.add(new Dimension(i));
		}
	}

	/**
	 * @return the m_individusMoyen
	 */
	public Vector<IndividuMoyen> getIndividusMoyens() {
		return m_individusMoyen;
	}
	
//	/**
//	 * @return the m_individusPartiels
//	 */
//	public Vector<IndividuPartiel> getIndividusPartiels() {
//		return m_individusPartiels;
//	}

	/**
	 * @return the m_variablesMoyenne
	 */
	public Vector<ContinueMoyenne> getVariablesMoyennes() {
		return m_continuesMoyennes;
	}

	/**
	 * @return the m_axesPartiels
	 */
	public Vector<AxePartiel> getAxesPartiels() {
		return m_axesPartiels;
	}

	/**
	 * Ajoute un individu moyen au jeu de donnees
	 * @param obj
	 */
	public void addIndividuMoyen(IndividuMoyen obj) {
		m_individusMoyen.add(obj);
	}
	
//	/**
//	 * Ajoute un individu partiel au jeu de donnees
//	 * @param obj
//	 */
//	public void addIndividuPartiel(IndividuPartiel obj) {
//		m_individusPartiels.add(obj);
//	}

	/**
	 * Ajoute un axe partiel au jeu de donnees
	 * @param obj
	 */
	public void addAxePartiel(AxePartiel obj) {
		m_axesPartiels.add(obj);
	}

	/**
	 * Ajoute une variable continue moyenne au jeu de donnees
	 * @param obj
	 */
	public void addContinueMoyenne(ContinueMoyenne obj) {
		m_continuesMoyennes.add(obj);
	}


	/**
	 * Ajoute un groupe au jeu de donnees
	 * @param obj
	 */
	public void addGroupe(Groupe obj) {
		m_groupes.add(obj);
	}


	/**
	 * Ajoute une variable nominale au jeu de donnees
	 * @param obj
	 */
	public void addNominale(Nominale obj){
		m_nominales.add(obj);
	}

	/**
	 * @return the m_groupes
	 */
	public Vector<Groupe> getGroupes() {
		return m_groupes;
	}

	/**
	 * Retrouve un groupe dans le jeu de donnees
	 * @param i le numero du groupe recherche
	 * @return le groupe si on le trouve, un nouveau groupe sinon
	 * @throws Exception 
	 */
	public Groupe getGroupe(int i) throws Exception {
		Groupe g=null;
		if(m_groupes==null)throw new Exception("pas de groupes dans le jeu de données");
		for(int j=0; j< m_groupes.size() && g==null;j++){
			Groupe g2=m_groupes.elementAt(j);
			if(g2.getNum() == i) g=g2;
		}
		if(g==null) {
			g=new Groupe(i);//TODO verifier s'il ne faut pas aussi ajouter le groupe au jeu de donnees !
			m_groupes.add(g);
		}
		return g;
	}

	/**
	 * @return the m_nominales
	 */
	public Vector<Nominale> getNominales() {
		return m_nominales;
	}

	/**
	 * Retrouve une nominale dans le jeu de donnees
	 * @param i le numero de la nominale recherchee
	 * @return la nominale si on la trouve, null sinon
	 * @throws Exception 
	 */
	public Nominale getNominale(int i) throws Exception
	{
		if(m_nominales==null)throw new Exception("pas de nominales dans le jeu de données");
		if(i<m_nominales.size() && i>=0)return m_nominales.get(i);
		else throw new Exception("nominale n° "+i+" introuvable");//TODO verifier s'il n'y a pas un comportement plus malin
	}

	/**
	 * Rend la nominale dont l'annotation correspond a celle passee en parametre
	 * rend null si elle n'existe pas
	 * @param annotation
	 * @return la nominale
	 */
	public Nominale getNominale(String annotation) {
		for (Nominale nom : m_nominales) {
			if(nom.getAnnotation().equals(annotation))return nom;
		}
		return null;
	}

	/**
	 * @return le tableau des pourcentages d'inertie des axes factoriels 
	 */
	public double[] getInerties() {
		return m_inerties;
	}

	/**
	 * fixe le tableau des pourcentages d'inertie des axes factoriels
	 * @param inerties
	 */
	public void setInerties(double[] inerties) {
		m_inerties = inerties;
	}

	/**
	 * Rend l'individu dont l'annotation correspond a celle passee en parametre
	 * rend null s'il n'existe pas
	 * @param annot
	 * @return l'individu
	 * @throws Exception 
	 */
	public Individu getIndividu(String annot) throws Exception{
		Individu ind=null;
		if(m_individusMoyen==null)throw new Exception("pas d'individus dans le jeu de données");
		for(int i=0 ; i<m_individusMoyen.size() && ind==null;i++){
			ind = m_individusMoyen.elementAt(i).getIndividu(annot);
		}
		return ind;
	}


	/**
	 * @param ind
	 * @param c
	 * @return la coordonnee de l'individu ind selon la variable continue c
	 */
	public double getValeur(Individu ind, ContinueAnalysee c) {
		return c.getValeur(ind);
	}

	/**
	 * @param mod
	 * @param c
	 * @return la coordonnee de la modalite mod selon la variable continue artificielle c
	 */

	public double getValeur(Modalite mod, ContinueArtificiellePourMod c){
		return c.getValeur(mod);
	}

	/**
	 * @param id
	 * @param c
	 * @return la valeur de la donnée id pour la continue c
	 */
	public double getValeur(DonneeID id, Continue c) {
		if(id instanceof Individu){
			return c.getValeur((Individu)id);
		}else if(id instanceof Modalite && c instanceof ContinueArtificiellePourMod){
			return ((ContinueArtificiellePourMod)c).getValeur((Modalite)id);
		}else if(id instanceof Modalite){
			return c.getValeur((Modalite)id);
		}else if(id instanceof Groupe){
			return c.getValeur((Groupe)id);
		}else if(id instanceof ContinueAnalysee && c instanceof ContinueArtificiellePourCont){
			return ((ContinueArtificiellePourCont)c).getValeur((ContinueAnalysee)id);
		}else throw new InvalidParameterException(id.getClass().getCanonicalName()+", "+c.getClass().getCanonicalName());
	}


	/**
	 * @param id
	 * @param c
	 * @param t
	 * @return la valeur de la donnée id pour la continue c
	 */
	public double getValeur(DonneeID id, Continue c, TypeAnalyse t)
    {
        return ((ContinueArtificiellePourCont)c).getValeur((ContinueAnalysee)id, t);
    }

	
	/**
	 * Rend la variable dont l'annotation correspond a celle passee en parametre
	 * rend null si elle n'existe pas
	 * @param annot
	 * @return la variable
	 */
	public ContinueAnalysee getContinue(String annot) {
		ContinueAnalysee var=null;
		for(int i=0 ; i<m_continuesMoyennes.size() && var==null;i++){
			var = m_continuesMoyennes.elementAt(i).getVariable(annot);
		}
		return var;
	}


	/**
	 * @param annot
	 * @return le numero (dans le tableau des noms de colonnes) de la variable dont l'annotation est passee en parametre
	 */
	public int getNumVariable(String annot)
	{
		for(int i = 0;i<getNbVar();i++)
		{
			String tmp = getVariablesMoyennes().elementAt(i).getAnnotation();
			if(annot.equals(tmp))return i;
		}
		return -1;
	}

	/**
	 * Rend la modalite dont l'annotation correspond a celle passee en parametre
	 * rend null si elle n'existe pas
	 * @param annot
	 * @return la modalite
	 */
	public Modalite getModalite(String annot) {
		Modalite mod=null;
		for(int i=0 ; i<m_nominales.size() && mod==null;i++){
			mod = m_nominales.elementAt(i).getModalite(annot);
		}
		return mod;
	}

	/**
	 * Methode qui dit si les donnees contiennent des individus actifs
	 * @return un booleen
	 */
	public boolean isIndActif() {
		boolean trouve=false;
		for(int i =0;i<m_individusMoyen.size() && !trouve;i++){
			trouve=m_individusMoyen.elementAt(i).isActive();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des individus illustratifs
	 * @return un booleen
	 */
	public boolean isIndIllus() {
		boolean trouve=false;
		for(int i =0;i<m_individusMoyen.size() && !trouve;i++){
			trouve=!m_individusMoyen.elementAt(i).isActive();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des modalites actives
	 * @return un booleen
	 */
	public boolean isModActif() {
		boolean trouve=false;
		for(int i =0;i<m_nominales.size() && !trouve;i++){
			trouve=m_nominales.elementAt(i).isModActif();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des modalites illustratives
	 * @return un booleen
	 */
	public boolean isModIllus() {
		boolean trouve=false;
		for(int i =0;i<m_nominales.size() && !trouve;i++){
			trouve=m_nominales.elementAt(i).isModIllus();
		}
		return trouve;
	}


	/**
	 * Methode qui dit si les donnees contiennent des groupes actifs
	 * @return un booleen
	 */
	public boolean isGroupeActif() {
		boolean trouve=false;
		for(int i =0;i<m_groupes.size() && !trouve;i++){
			trouve=m_groupes.elementAt(i).isActive();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des groupes illustratifs
	 * @return un booleen
	 */
	public boolean isGroupeIllus() {
		boolean trouve=false;
		for(int i =0;i<m_groupes.size() && !trouve;i++){
			trouve=!m_groupes.elementAt(i).isActive();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des variales actives orthonormees
	 * @return un booleen
	 */
	public boolean isVarActifOrtho() {
		boolean trouve=false;
		for(int i =0;i<m_continuesMoyennes.size() && !trouve;i++){
			trouve=m_continuesMoyennes.elementAt(i).isActive()
			&& m_continuesMoyennes.elementAt(i).isOrthonorme();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des variales illustratives orthonormees
	 * @return un booleen
	 */
	public boolean isVarIllusOrtho() {
		boolean trouve=false;
		for(int i =0;i<m_continuesMoyennes.size() && !trouve;i++){
			trouve=!m_continuesMoyennes.elementAt(i).isActive()
			&& m_continuesMoyennes.elementAt(i).isOrthonorme();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des variales actives non orthonormees
	 * @return un booleen
	 */
	public boolean isVarActifNonOrtho() {
		boolean trouve=false;
		for(int i =0;i<m_continuesMoyennes.size() && !trouve;i++){
			trouve=m_continuesMoyennes.elementAt(i).isActive()
			&& !m_continuesMoyennes.elementAt(i).isOrthonorme();
		}
		return trouve;
	}

	/**
	 * Methode qui dit si les donnees contiennent des variales illustratives non orthonormees
	 * @return un booleen
	 */
	public boolean isVarIllusNonOrtho() {
		boolean trouve=false;
		for(int i =0;i<m_continuesMoyennes.size() && !trouve;i++){
			trouve=!m_continuesMoyennes.elementAt(i).isActive()
			&& !m_continuesMoyennes.elementAt(i).isOrthonorme();
		}
		return trouve;
	}

	/**
	 * @return un booleen
	 */
	public boolean isVar() {
		return m_continuesMoyennes.size()>0;
	}

	/**
	 * Methode qui dit si les donnees contiennent des individus moyens
	 * @return un booleen
	 */
	public boolean isIndMoy() {
		boolean trouve=false;
		if (m_individusMoyen!= null) {trouve=true;}
		return trouve;
	}
	
	/**
	 * Methode qui dit si les donnees contiennent des individus partiels
	 * @return un booleen
	 */
	public boolean isIndPart() {
		boolean trouve=false;
		for(int i =0;i<m_individusMoyen.size() && !trouve;i++){
			trouve=m_individusMoyen.elementAt(i).getNbBranches() >0;
		}
		return trouve;
	}


	/**
	 * Methode qui dit si les donnees contiennent des variables (continues) partielless
	 * @return un booleen
	 */
	public boolean isVarPart() {
		boolean trouve=false;
		for(int i =0;i<m_continuesMoyennes.size() && !trouve;i++){
			trouve=m_continuesMoyennes.elementAt(i).getNbBranches() >0;
		}
		return trouve;
	}

	/**
	 * @return vrai si les données contiennent des axes partiels
	 */
	public boolean isAxesPart(){
		return m_axesPartiels.size()>0;
	}
	
	/**
	 * @return vrai s'il est possible de représenter un dataframe
	 */
	public boolean isRepDataframePossible(){
		return getVariablesMoyennes()!=null && getVariablesMoyennes().size()>1;
	}

	/**
	 * @return le nombre d'individus (moyens et partiels)
	 */
	
	// @trovalet
	public int getNbIndividu() {
		int taille=0;
		for (IndividuMoyen ind : m_individusMoyen) {
			taille+=ind.getNbBranches()+1;
		}
		return taille;
	}

	/**
	 * @return le nombre de variables continues moyennes
	 */
	public int getNbVar() {
		return m_continuesMoyennes.size();
	}

	/**
	 * Verifie l'integrite d'un jeu de donnees, et essaie de les reparer si possible
	 * @throws ExceptionIntegrite en cas d'erreur impossible a corriger
	 */
	public void verifierIntegrite() throws ExceptionIntegrite {
		for (Nominale nom : m_nominales) {
			nom.verifierIntegrite();
		}
		for (Individu ind : m_individusMoyen) {
			ind.verifierIntegrite();
		}
		
//		if (m_individusPartiels!= null) {
//			System.out.println("vecteur m_individusPartiels non nul");
//			for (IndividuPartiel indPartiel : m_individusPartiels) {
//				indPartiel.verifierIntegrite();
//			}
//		}
		

		boolean[] numeros=new boolean[m_continuesMoyennes.size()];
		for (int i = 0; i < numeros.length; i++) {
			numeros[i]=false;
		}
		for(int j=0;j<m_continuesMoyennes.size();j++) {
			ContinueMoyenne var= m_continuesMoyennes.get(j);
			try {
				var.verifierIntegrite();
			}catch (ExceptionIntegrite e) {
				if(e.getMessage().equals(ExceptionIntegrite.errNumCont)) {
					if(var.getNum()<0 || getNumVariable(var.getAnnotation())!=0) {
						//Il y a bien erreur : on fixe le bon numero
						var.setNum(j);
						try{
							var.verifierIntegrite();
						}catch(ExceptionIntegrite e2){
							if(!e.getMessage().equals(ExceptionIntegrite.errNumCont) || var.getNum()!=0) throw e2;
						}
					}
				}else throw e;    
			}
			if(numeros[var.getNum()]) throw new ExceptionIntegrite("numeros de variables redondants");
			else numeros[var.getNum()]=true;
		}
		for (Groupe gr : m_groupes) {
			gr.verifierIntegrite();			
			//vérification du nombre d'axes partiels
			int nbAxesPartiels=0;
			for (ContinueAnalysee axe : m_axesPartiels) {
				if(axe instanceof AxePartiel){
					if(axe.getGr().equals(gr)) nbAxesPartiels++;
					((AxePartiel)axe).verifierIntegrite();
				}else{
					throw new ExceptionIntegrite("Des continues normales ont été rangées avec les axes partiels : \n"+axe.toString());
				}
			}
			if(nbAxesPartiels > getNbAxe()){
				throw new ExceptionIntegrite("nombre anormal d'axes partiels sur le groupe "+gr.getNum()+" ("+gr.getNum()+") : "+nbAxesPartiels+" au lieu de max "+getNbAxe());
			}
		}
		if(getNbAxe()>getInerties().length)setNbAxe(getInerties().length);

	}

	/** creation d'une collection de DonneeID a partir des DonneeParse 
	 * @return un vector*/
	public Vector<DonneeID> toDonneeID()
	{
		Vector<DonneeID> liste=new Vector<DonneeID>();
		for(int i=0;i<getIndividusMoyens().size();i++)
			liste.add(getIndividusMoyens().get(i));
		for(int j=0;j<getVariablesMoyennes().size();j++)
			liste.add(getVariablesMoyennes().get(j));
		for(int k=0;k<getAxesPartiels().size();k++)
			liste.add(getAxesPartiels().get(k));	
		liste.addAll(getModalites());
		for(int l=0;l<getGroupes().size();l++){
			try {
				if(getGroupe(l).getNum()!=0)
					liste.add(getGroupe(l));
			} catch (Exception e) {
				//N'arrivera pas, par construction
				new FenetreErreur(e);
			}
		}
		return liste;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		String chaine="";
		try {
			chaine="Jeu de données :\n"+m_individusMoyen.size()+" individus :\n";
			for (IndividuMoyen ind : m_individusMoyen) {
				chaine+=ind.toString()+"\n";
			}
			chaine+="================\nVariables quantitatives :\n";
			for (ContinueMoyenne var : m_continuesMoyennes) {
				chaine+=var.toString()+"\n";
			}
			chaine+="================\nVariables qualitatives :\n";
			for (Nominale nom : m_nominales) {
				chaine+=nom.toString()+"\n";
			}
			chaine+="================\nGroupes ("+m_groupes.size()+"):\n";
			for (Groupe groupe : m_groupes) {
				chaine+=groupe.toString()+"\n";
			}
			
			chaine+="====================\nNombre d'axes : "+getNbAxe();
			chaine+="\nliste : ";
			for (Dimension axe : m_axes) {
				chaine+=axe.toString()+" ";
			}
			chaine+="\ninerties :";
			for(int i=1;i<=getNbAxe();i++){
				chaine+="\naxe "+i+" : "+m_inerties[i-1]+" %";
			}
			chaine+="====================\nNombre d'axes partiels: "+m_axesPartiels.size();
			chaine+="\nliste :";
			for (ContinueAnalysee var : m_axesPartiels) {
				chaine+=var.toString()+"\n";
			}
		}catch (Exception e) {
			chaine+="\nERREUR :\n"+e;
		}
		return chaine;
	}


	/**
	 * formate la chaine
	 * @param d
	 * @return une chaide charactere
	 */
	public static String pourcentage(double d) {
		return String.format("%1$.5s",d)+" %";
	}

	/**
	 * @return the m_type
	 */
	public TypeAnalyse getType() {
		return m_type;
	}

	/**
	 * @param m_type the m_type to set
	 */
	public void setType(TypeAnalyse m_type) {
		this.m_type = m_type;
	}

	/**
	 * @return la liste des modalités
	 */
	public Vector<ModaliteMoyenne> getModalites() {
		Vector<ModaliteMoyenne> liste=new Vector<ModaliteMoyenne>();
		for(int i=0;i<getNominales().size();i++)
		{
			Vector<ModaliteMoyenne> mod=getNominales().get(i).getModalites();
			for(int j=0;j<mod.size();j++)
				liste.add(mod.get(j));
		}
		return liste;
	}

	/**
	 * @param n
	 * @return l'axe numéro n (n>=0)
	 */
	public Dimension getAxe(int n) {
		for (Dimension axe : m_axes) {
			if(axe.getNum()==n)return axe;
		}
		return null;
	}

	/**
	 * @return un booleen
	 */
	public boolean isNominales() {
		return getNominales().size()>0;
	}



}
