package don;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Vector;

import exceptions.ExceptionIntegrite;

/**
 * Variable nominale/qualitative
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public class Nominale implements Serializable, Variable, Verifiable, Qualifiable{

	private static final long serialVersionUID = 6606347683598577331L;

	/**
	 * Les différentes valeurs/modalités de la variable
	 */
	/** SLE : vecteur des modalités associées à une variable nominale */
	private Vector<ModaliteMoyenne> m_modalites;

	/**
	 * Le nom de la variable
	 * SLE : Le nom de la variable nominale
	 */
	private String m_nom;
	
	/** le tableau des modalités qui sont issues du demod **/
	/** SLE : pour une modalité donnée (à vérifier) tableau des modalités qui la caractérisent et des valeurs tests associées */    
	private HashMap<Modalite,Double> val_descMod;
	
	/** le tableau des  **/
	/** SLE : pour une modalité donnée (à vérifier) tableau des variables continues qui la caractérisent et des valeurs tests associées */
	private HashMap<ContinueAnalysee,Double> val_descCont;
	
	/** le tableau des **/
	/** SLE : pour une nominale donnée (à vérifier) tableau des variables nominlaes qui la caractérisent et des valeurs tests associées */
	private HashMap<Nominale,Double> val_descNom;

	/**
	 * @param nom
	 */
	public Nominale(String nom) {
		m_nom = nom;
		m_modalites = new Vector<ModaliteMoyenne>();
		val_descMod=new HashMap<Modalite,Double>();
		val_descCont=new HashMap<ContinueAnalysee, Double>();
		val_descNom=new HashMap<Nominale, Double>();
	}

	/**
	 * @return the m_nom
	 */
	public String getAnnotation() {
		return m_nom;
	}

	/**
	 * @param nom the m_nom to set
	 */
	public void setNom(String nom) {
		m_nom = nom;
	}

	/**
	 * @return the m_modalites
	 */
	public Vector<ModaliteMoyenne> getModalites() {
		return m_modalites;
	}

	/**
	 * Ajoute une modalité moyenne à la nominale
	 * @param mod
	 */
	public void addModaliteMoyenne(ModaliteMoyenne mod) {
		m_modalites.add(mod);
		if(mod.getNominale()==null || !mod.getNominale().equals(this)){
			//si l'individu n'est pas au courant, l'informer
			mod.setNominale(this);
		}
	}

	/** 
	 * @return le nombre de modalités de la nominale
	 */
	public int getNbModalites() {
		return m_modalites.size();
	}

	/**
	 * Rend la modalité dont l'annotation correspond à celle passée en paramètre
	 * rend null si elle n'existe pas
	 * @param annot
	 * @return la modalité
	 */
	public Modalite getModalite(String annot) {
		Modalite mod=null;
		for(int i=0 ; i<m_modalites.size() && mod==null;i++){
			mod = m_modalites.elementAt(i).getModalite(annot);
		}
		return mod;
	}

	/**
	 * Dit si la nominale contient des modalités actives 
	 * @return un booléen
	 */
	public boolean isModActif() {
		boolean trouve=false;
		for(int i =0;i<m_modalites.size() && !trouve;i++){
			trouve=m_modalites.elementAt(i).isActive();
		}
		return trouve;
	}

	/**
	 * Dit si la nominale contient des modalités illustratives 
	 * @return un booléen
	 */
	public boolean isModIllus() {
		boolean trouve=false;
		for(int i =0;i<m_modalites.size() && !trouve;i++){
			trouve=!m_modalites.elementAt(i).isActive();
		}
		return trouve;
	}

	/**
	 * @param modalite
	 * @return la position de la modalité dans la liste de modalités, -1 si elle n'est pas trouvée
	 */
	private int position(Modalite modalite)
	{return m_modalites.indexOf(modalite);}

	/**
	 * @param don 
	 * @return la position de la modalité de la donnée pour la variable dans la liste de modalités, -1 si elle n'est pas trouvée
	 */
	public int getNumModalite(DonneeID don){
		if(don instanceof Individu)
		{
			Individu ind=(Individu) don;
			Modalite mod=ind.getModalite(this);
			if(mod!=null)
				return position(mod);
			else
				return -1;
		}else
			return -1;
		
	}

	
	public double getDescMod(Modalite mod){
		Double val = val_descMod.get(mod);
		if(val==null)return 0;
		else return val;
	}

	
	public void setDescMod(Modalite mod, Double vtest) {
		val_descMod.put(mod, vtest);
	}

	
	public double getDescCont(ContinueAnalysee cont){
		Double val = val_descCont.get(cont);
		if(val==null)return 0;
		else return val;
	}

	public void setDescCont(ContinueAnalysee cont, Double vtest) {
		val_descCont.put(cont, vtest);
	}
	
	public double getDescNom(Nominale nom){
		Double val = val_descNom.get(nom);
		if(val==null)return 0;
		else return val;
	}

	public void setDescNom(Nominale nom, Double vtest) {
		val_descNom.put(nom, vtest);
	}
	
	public double getDesc(DonneeID don){
		if(don instanceof Modalite){
			return getDescMod((Modalite)don);
		}else if(don instanceof ContinueAnalysee){
			return getDescCont((ContinueAnalysee)don);
		}else return 0;
	}
	
	public double getDesc(Nominale don){
		return getDescNom(don);
	}
	
	
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String chaine="---\nvariable nominale : "+m_nom+" \nmodalités : \n";
		for (ModaliteMoyenne mod : m_modalites) {
			chaine+=mod.toString()+"\n";
		}
		chaine+="\n v-tests pour les modalites :\n";
		for (QualifiableRepresentable mod : val_descMod.keySet()) {
			chaine+=mod.getAnnotation()+" : "+val_descMod.get(mod)+" | ";
		}
		chaine+="\n v-tests pour les continues :\n";
		for (Continue cont : val_descCont.keySet()) {
			chaine+=cont.getAnnotation()+" : "+val_descCont.get(cont)+" | ";
		}
		chaine+="\n v-tests pour les nominales :\n";
		for (Nominale nom : val_descNom.keySet()) {
			chaine+=nom.getAnnotation()+" : "+val_descNom.get(nom)+" | ";
		}
		return chaine;
	}



	public void verifierIntegrite() throws ExceptionIntegrite {
		if(m_nom==null)throw new ExceptionIntegrite("nominale sans nom");
		for (Modalite mod : m_modalites) {
			if(mod.getNominale()==null)mod.setNominale(this);
			else if(mod.getNominale()!=this)throw new ExceptionIntegrite("incohérence nominale - modalité");
			mod.verifierIntegrite();
		}
	}

}
