package dyn;

import ihm.FenetreErreur;
import ihm.Gestionnaire;
import ihm.Habilleur;
import ihm.IHM;

import java.awt.Color;
import java.awt.Shape;
import java.util.HashMap;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.Vector;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYIDAnnotation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.AbstractXYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;

import outils.Constantes;
import actions.PanelMouseActionListener;
import don.Continue;
import don.DonneeID;
import don.Groupe;
import don.Individu;
import don.Modalite;
import don.Nominale;
import don.Variable;
import don.DonneeParse.TypeAnalyse;
import drawing.Canvas;
import drawing.Couleur;
import drawing.Series;
import exceptions.ExceptionIntegrite;

/** Classe qui regroupe les principaux points communs entre les graphes de dynGraph
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 */
public abstract class Graphe
{
	/** l'orientation du graphique 
	 * par defaut : VERTICAL
	 */
	public static final PlotOrientation DEFAULT_ORIENTATION = PlotOrientation.VERTICAL;

	/** les données courantes */
	private SortedSet<Series> m_donCourante;

	/** l'orientation du plot */
	private PlotOrientation m_orientation;

	/** le titre du graphe */
	private String m_titreGraphe;

	/** le titre de l'axe X */
	private String m_titreX;

	/** le titre de l'axe Y */
	private String m_titreY;	

	/** le Gestionnaire qui commande le graphe */
	protected Gestionnaire m_gestionnaire;

	/** le canevas sur lequel on dessine */
	protected Canvas m_canvas;

	/** le jeu de données sous forme jfreechart */
	protected XYDataset m_data;

	protected Habilleur m_habilleur;

	private HashMap<Nominale, Color> m_tabCoulNom;

	private int m_nbSeriesTraitees;

	/** Constructeur
	 * @param gest : le Gestionnaire qui commande le graphe  
	 * @param hab : l'habilleur qui commande la présentation du graphe
	 * @param titreGraphe : le titre du graphe 
	 * @param titreX : le titre de l'axe X 
	 * @param titreY : le titre de l'axe Y
	 */
	public Graphe(Gestionnaire gest, Habilleur hab, String titreGraphe, String titreX, String titreY)
	{
		m_gestionnaire=gest;
		m_habilleur=hab;
		m_titreGraphe=titreGraphe;m_titreX=titreX;m_titreY=titreY;
		m_orientation=DEFAULT_ORIENTATION;
	}

	/** met a jour l'affichage de la grille, 
	 * interroge le Gestionnaire pour savoir si elle doit étre affichée
	 */
	public void affGrid()
	{
		XYPlot plot = (XYPlot) m_gestionnaire.getPanel().getChart().getPlot();
		if(m_gestionnaire.isAffGrid())
		{
			plot.setDomainGridlinesVisible(true);
			plot.setRangeGridlinesVisible(true);
		}
		else
		{
			plot.setDomainGridlinesVisible(false);
			plot.setRangeGridlinesVisible(false);
		}
	}

	/** met a jour l'affichage des annotations d'une liste, 
	 * interroge le Gestionnaire pour savoir si elles doivent étre affichées
	 * @param liste : la liste de données dont les annotations doivent étre mises é jour
	 */
	protected void affAnnotation(Vector<DonneeID> liste)
	{
		setCanvas();
		XYPlot plot = (XYPlot) m_gestionnaire.getPanel().getChart().getPlot();
		AbstractXYItemRenderer renderer = (AbstractXYItemRenderer) plot.getRenderer();
		if(m_gestionnaire.getCanvas()!=null)
			m_gestionnaire.getCanvas().clearAnnot();
		renderer.removeAnnotations();
		if(m_data.getSeriesCount()!=0)
		{
			for (DonneeID id : liste)
			{
				if(m_gestionnaire.isAnnotations(id) && !m_gestionnaire.isFantome(id))
				{
					double x,y;
					Continue axeX=m_gestionnaire.getAxeX();
					Continue axeY=m_gestionnaire.getAxeY();
					x=id.getCoord(axeX);
					y=id.getCoord(axeY);
					XYIDAnnotation annot = new XYIDAnnotation(id.getAnnotation(),id.getXannot(axeX,axeY),id.getYannot(axeX,axeY), x, y,getCouleur(id));
					m_habilleur.setFont(annot, id);
					annot.setTextAnchor(id.getAnchor());
					//annot.setToolTipText(id.toString());
					renderer.addAnnotation(annot);
					m_canvas.addAnnot(annot);
				}
			}
		}
	}

	/** met a jour l'affichage des annotations du graphe, 
	 * interroge le Gestionnaire pour savoir si elles doivent étre affichées
	 */
	public void affAnnotation()
	{
		Vector<DonneeID> liste=new Vector<DonneeID>();
		Iterator<Series> ite=getDonnee().iterator();
		while(ite.hasNext())
		{
			Series s=ite.next();
			for(int i=0;i<s.getNbDonneeID();i++)
				liste.add(s.getDonneeID(i));
		}
		affAnnotation(liste);
	}

	/** retrouve la couleur d'une donnée en recherchant la couleur de sa série
	 * @param obj : la donnée dont on veut trouver la couleur
	 * @return la couleur de la donnée, rouge si on ne la trouve pas
	 */
	protected Color getCouleur(DonneeID obj)
	{
		Iterator<Series> ite=getDonnee().iterator();
		while(ite.hasNext())
		{
			Series s=ite.next();
			for(int i=0;i<s.getNbDonneeID();i++)
			{
				if(s.getDonneeID(i).equals(obj))
					return (Color)s.getColor();
			}
		}
		new FenetreErreur("impossible de trouver la série de "+obj);
		return Color.red;
	}

	/** met a jour le graphique en consultant le gestionnaire
	 * @param series : le jeu de données a utiliser
	 */
	public void update(SortedSet<Series> series)
	{
		try
		{
			//création du panel
			ChartPanel chartPanel = m_gestionnaire.getPanel();
			if(chartPanel==null)
			{
				chartPanel=new ChartPanel(new JFreeChart(new XYPlot()));
				chartPanel.addChartMouseListener(new PanelMouseActionListener(m_gestionnaire,chartPanel));
				m_gestionnaire.setPanel(chartPanel);
			}
			else
				updateCanvas();

			//mise a jour des données
			setDonnee(series);
			m_data = construireSeries();
			metEnFormeSeries();

			//Création du graphe
			JFreeChart chart = createChart(m_data);
			chart.setRapport(m_gestionnaire.getRapport());
			//mise à jour panel
			chartPanel.setChart(chart);
			//format des nombres
			XYPlot plot = (XYPlot) chart.getPlot();
			((NumberAxis)(plot.getDomainAxis())).setNumberFormatOverride(Constantes.FORMAT_NOMBRES);
			//configuration du zoom
			chartPanel.setDomainZoomable(true);
			chartPanel.setRangeZoomable(true);
			//fin de la configuration du panel

			//repère normé, centré ?
			plot.setRangeZeroBaselineVisible(true);
			plot.useEqualAxes(m_gestionnaire.isOrtho());
			int type = m_gestionnaire.getType();
			plot.setUseSquaredAndCenteredArea(type==Gestionnaire.GROUPES || type==Gestionnaire.VARIABLES || type == Gestionnaire.AXES_PARTIELS);

			//mise é jour du canevas
			m_gestionnaire.setCanvas(m_canvas);

			//mise é jour du rendu
			affAnnotation();
			affGrid();
			renderer();


		}
		catch (Exception e)
		{
			new FenetreErreur(e);
		}
	}

	/** Construit le jeu de données jfreechart é partir des données courantes
	 * @return un jeu de donnee jfreechart
	 */
	protected abstract XYDataset construireSeries();

	/** création du graphique	 
	 * @param dataset : le jeu de données é représenter
	 * @return le graphique correspondant, null s'il y a une erreur dans les paramétres
	 */
	protected abstract JFreeChart createChart(XYDataset dataset);

	/** met a jour le canevas : 
	 * on le récupére sur le Gestionnaire s'il en a un, sinon on en crée un 
	 */
	protected boolean setCanvas()
	{
		if(m_gestionnaire!=null)
		{
			if(m_gestionnaire.getCanvas()==null) m_gestionnaire.setCanvas(new Canvas());
			m_canvas=m_gestionnaire.getCanvas();
			if(m_canvas==null)return false;
			else return true;
		}
		else return false;
	}

	/** met a jour le contenu du canevas */
	protected boolean updateCanvas()
	{
		if(m_canvas!=null)
		{
			m_canvas.clearAnnot();
			m_canvas.clearBranche();
			m_canvas.clearPoint();
			return true;
		}
		return false;
	}

	/** @return le titre du graphe */
	protected String getTitreGraphe()
	{return m_titreGraphe;}

	/** Fixe le titre du graphe
	 * @param titreGraphe
	 */
	protected void setTitreGraphe(String titreGraphe)
	{m_titreGraphe=titreGraphe;}

	/** @return le titre de l'axe X */
	protected String getTitreX()
	{return m_titreX;}

	/** Fixe le titre de l'axe X
	 * @param titreX
	 */
	protected void setTitreX(String titreX)
	{m_titreX=titreX;}

	/** @return le titre de l'axe Y */
	protected String getTitreY()
	{return m_titreY;}

	/** Fixe le titre de l'axe Y
	 * @param titreY
	 */
	protected void setTitreY(String titreY)
	{m_titreY=titreY;}

	/** @return les données courantes */
	protected SortedSet<Series> getDonnee()
	{return m_donCourante;}

	/** Fixe les données
	 * @param series la table légende -> série
	 * @throws ExceptionIntegrite 
	 */
	protected void setDonnee(SortedSet<Series> series) throws ExceptionIntegrite
	{
		Iterator<Series> ite=series.iterator();
		Vector<String> labels = new Vector<String>();
		
		while(ite.hasNext()){
			Series serie = ite.next();
			String label = serie.getSemantique();
			if(!labels.contains(label)){
				labels.add(label);
			}else throw new ExceptionIntegrite("Nom de série redondant : "+label);
		}
		
		
		m_donCourante=series;
	}

	/** Fixe l'orientation
	 * @param orientation
	 */
	protected void setOrientation(PlotOrientation orientation)
	{m_orientation = orientation;}

	/** @return l'orientation */
	protected PlotOrientation getOrientation()
	{return m_orientation;}

	/** Fixe le rendu en paramétrant l'objet "renderer" du plot */
	protected void renderer()
	{
		XYPlot plot = (XYPlot) m_gestionnaire.getPanel().getChart().getPlot();
		AbstractXYItemRenderer renderer = (AbstractXYItemRenderer) plot.getRenderer();
		Iterator<Series> ite=getDonnee().iterator();
		int nbSerie=0;
		while(ite.hasNext())
		{
			Series s=ite.next();
			if(!s.isFantome(m_gestionnaire)) {
				renderer.setSeriesPaint(nbSerie, s.getColor());
				renderer.setSeriesShape(nbSerie, s.getForme());
				renderer.setSeriesVisibleInLegend(nbSerie, s.isAffLegende());
				//donneFormeSerie(s, nbSerie, renderer);
				if(renderer instanceof XYLineAndShapeRenderer){
					((XYLineAndShapeRenderer)renderer).setSeriesShapesFilled(nbSerie,s.isFormePleine());
				}
				nbSerie++;
			}
		}
	}

	/** détermine les couleurs et les formes des séries */
	protected void metEnFormeSeries()
	{
		m_nbSeriesTraitees=0;
		Iterator<Series> ite=getDonnee().iterator();
		Couleur c=new Couleur();
		m_tabCoulNom=new HashMap<Nominale, Color>();
		while(ite.hasNext())
		{
			Series s=ite.next();
			DonneeID don=s.getDonneeID(0);
			//On prend la première donnée de la série, 
			//et on considère qu'elle est représentative
			//C'est un peu cavalier mais bon...

			s.setForme(donneFormeSerie(don));
			s.setColor(donneCouleurSerie(c, s, don));
			s.setAffLegende(donneAffLegendeSerie(s, don));
		}
	}

	private boolean donneAffLegendeSerie(Series s, DonneeID don) {
		boolean setAffLegend;

		String cde=s.getSemantique();
		Continue habillageContinu = m_habilleur.getHabillageContinu(don);

		if(cde.contains(String.valueOf(IHM.getXmlLangue().hashCode()))
				|| habillageContinu!=null && m_habilleur.isHabillageCentre(don)
				|| (m_gestionnaire.getType()!=Gestionnaire.DATAFRAME && m_habilleur.isSousSeuilAffinage(don))){
			setAffLegend=false;
		}else setAffLegend=true;

		return setAffLegend;
	}

	private Color donneCouleurSerie(Couleur c, Series s, DonneeID don) {
		Color couleur;
		String cde=s.getSemantique();
		Continue habillageContinu = m_habilleur.getHabillageContinu(don);
		if(cde.contains(String.valueOf(IHM.getXmlLangue().hashCode()))){
			couleur=Constantes.COULEUR_SELECTION;
			s.setAffLegende(false);
		}else if(m_gestionnaire.getType()!=Gestionnaire.DATAFRAME && m_habilleur.isSousSeuilAffinage(don)){
			couleur=Constantes.COULEUR_HABILLAGE_CONTINUE_ZERO;
		}
		else if(habillageContinu!=null && m_habilleur.isHabillageCentre(don)){
			if(habillageContinu.getValeur(don)>0){
				couleur=Constantes.COULEUR_HABILLAGE_CONTINUE_POSITIF;
			}else if(habillageContinu.getValeur(don)<0){
				couleur=Constantes.COULEUR_HABILLAGE_CONTINUE_NEGATIF;
			}else{
				couleur=Constantes.COULEUR_HABILLAGE_CONTINUE_ZERO;
			}				
			s.setAffLegende(false);
		}
		else
		{
			couleur=c.getColor(m_nbSeriesTraitees);
			Variable habillageDiscret = m_habilleur.getHabillageDiscret(don);
			if(don instanceof Variable) {
				if(habillageDiscret==null) {
					if(don.isActive())
						couleur=Constantes.COULEUR_VARIABLES_ACTIVES;
					else
						couleur=Constantes.COULEUR_VARIABLES_ILLUSTRATIVES;
				}else couleur=c.getColor(don.getGr().getNum());
			}
			else if(don instanceof Individu && habillageDiscret==null 
					&& 
					!(habillageContinu!=null && m_habilleur.getHabillageStyleContinu(don)==Habilleur.STYLE_BULLES))
				if(don.isActive())couleur=Constantes.COULEUR_INDIVIDUS_ACTIFS;
				else couleur=Constantes.COULEUR_INDIVIDUS_ILLUSTRATIFS;
			else if( don instanceof Groupe)
				if(don.isActive())couleur=Constantes.COULEUR_GROUPES_ACTIFS;
				else couleur=Constantes.COULEUR_GROUPES_ILLUSTRATIFS;
			else if(don instanceof Modalite) {
				Nominale nom=((Modalite)don).getNominale();
				if(m_tabCoulNom.containsKey(nom)) {
					couleur=m_tabCoulNom.get(nom);
				}else {
					if(m_gestionnaire.getDon().getType() == TypeAnalyse.CA){
						if(don.isActive()){
							if(nom.getAnnotation().equals("row")){
								couleur=Constantes.COULEUR_AC_LIGNE_ACTIVE;
							}else{
								couleur=Constantes.COULEUR_AC_COLONNE_ACTIVE;
							}
						}else{
							if(nom.getAnnotation().equals("row")){
								couleur=Constantes.COULEUR_AC_LIGNE_ILLUSTRATIVE;
							}else{
								couleur=Constantes.COULEUR_AC_COLONNE_ILLUSTRATIVE;
							}
						}

					}else{
						couleur=c.getColor(m_nbSeriesTraitees);
						m_nbSeriesTraitees++;
						m_tabCoulNom.put(nom, couleur);
					}
				}

			}else m_nbSeriesTraitees++;
		}
		return couleur;
	}

	private Shape donneFormeSerie(DonneeID don) {
		Shape forme;
		if(m_gestionnaire.getType()!=Gestionnaire.DATAFRAME && m_habilleur.isSousSeuilAffinage(don)){
			forme= Constantes.FORME_ELEMENT_SOUS_SEUIL_AFFINAGE;
		}else if(don instanceof Individu)
			forme= Constantes.FORME_INDIVIDUS;
		else if(don instanceof Modalite)
			forme= Constantes.FORME_MODALITES;
		else if(don instanceof Groupe) {
			if(don.isActive())
				forme= Constantes.FORME_GROUPES_ACTIFS;
			else
				forme= Constantes.FORME_GROUPES_ILLUSTRATIFS;
		}else forme= Constantes.FORME_INDIVIDUS;
		return forme;
	}
}
