/* ===========================================================
 * JFreeChart : a free chart library for the Java(tm) platform
 * ===========================================================
 *
 * (C) Copyright 2004, by Michel Santos
 *
 * Project Info:  http://www.jfree.org/jfreechart/index.html
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 * Created on January 2, 2004, 2:27 PM
 */

package org.jfree.chart.plot.util;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * A class intended for very easy plotting of XY data
 * 
 * @author Michel Santos
 * @since 0.9.20
 */
public class SimpleXYChart {

	public SimpleXYChart() {
		init();
	}

	private JFreeChart chart;

	public JFreeChart getChart() {
		return chart;
	}

	private void init() {
		String strTitle = "";
		String strXLabel = "";
		String strYLabel = "";
		XYSeriesCollection colxn = null;
		chart = createLineShapePlot(strTitle, strXLabel, strYLabel, colxn,
				org.jfree.chart.plot.PlotOrientation.VERTICAL, false, false,
				false);
		chart.setBackgroundPaint(java.awt.Color.WHITE);
	}

	public void plot(double[] x, double[] y) {
		// Safety-check
		if (x == null || y == null) {
			throw new IllegalArgumentException(
					"Valid X and Y arrays should be provided");
		}
		// Safety-check
		if (x.length != y.length) {
			throw new IllegalArgumentException(
					"X and Y should be of the same length");
		}

		// Get the current data collection
		XYPlot plot = chart.getXYPlot();
		XYSeriesCollection colxn = (XYSeriesCollection) plot.getDataset();
		// XYSeriesCollection colxn = null; // 2005-05-26
		if (colxn == null) {
			colxn = new XYSeriesCollection();
		}
		int iNbrSeries = colxn.getSeriesCount() - 1 + 1;

		// Create the new data series
		// XYSeries data = new XYSeries("Series " + iNbrSeries); // NEW
		// data.isSorted(false);
		XYSeries data = new XYSeries("Series " + iNbrSeries, false, true); // NEW
		int iEntries = x.length;
		for (int i = 0; i < iEntries; i++) {
			data.add(x[i], y[i]);
		}

		// Set the new data collection
		colxn.addSeries(data);
		plot.setDataset(colxn);
	}

	public void plot(double[] x, double[] y, String label) {
		plot(x, y, null, null, label);
	}

	public void plot(double[] x, double[] y, java.awt.Paint paint) {
		plot(x, y, paint, null, null);
	}

	public void plot(double[] x, double[] y, java.awt.Paint paint, String label) {
		plot(x, y, paint, null, label);
	}

	public void plot(double[] x, double[] y, java.awt.Paint paint,
			java.awt.Shape shape) {
		plot(x, y, paint, shape, false, null);
	}

	public void plot(double[] x, double[] y, java.awt.Paint paint,
			java.awt.Shape shape, String label) {
		plot(x, y, paint, shape, false, label);
	}

	public void plot(double[] x, double[] y, java.awt.Paint paint,
			java.awt.Shape shape, boolean withoutLines, String label) {
		// Safety-check
		if (x == null || y == null) {
			throw new IllegalArgumentException(
					"Valid X and Y arrays should be provided");
		}
		// Safety-check
		if (x.length != y.length) {
			throw new IllegalArgumentException(
					"X and Y should be of the same length");
		}

		// Create data series
		// XYSeries data = new XYSeries(label); // NEW
		// data.isSorted(false);
		if (label == null) {
			label = "{Unlabelled}";
		}
		XYSeries data = new XYSeries(label, false, true); // NEW
		int iEntries = x.length;
		for (int i = 0; i < iEntries; i++) {
			data.add(x[i], y[i]);
		}

		// Create renderer (shape and color)
		StandardXYItemRenderer renderer;
		if (shape == null) {
			renderer = new StandardXYItemRenderer(StandardXYItemRenderer.LINES);
		} else {
			if (withoutLines) {
				renderer = new StandardXYItemRenderer(
						StandardXYItemRenderer.SHAPES);
			} else {
				renderer = new StandardXYItemRenderer(
						StandardXYItemRenderer.SHAPES_AND_LINES);
			}
			renderer.setShape(shape);
		}
		// renderer.setShapesFilled(false);
		renderer.setShapesFilled(true);
		renderer.setPaint(paint);

		// Get the current data collection
		XYPlot plot = chart.getXYPlot();
		// int iNbrSeries = plot.getSecondaryDatasetCount();
		int iNbrSeries = plot.getDatasetCount();

		// Set the new data collection
		// int iNewSeries = iNbrSeries - 1 + 1;
		int iNewSeries = iNbrSeries + 1;
		// plot.setSecondaryRenderer(iNewSeries, renderer);
		// plot.setSecondaryDataset(iNewSeries, new XYSeriesCollection(data));
		plot.setRenderer(iNewSeries, renderer);
		plot.setDataset(iNewSeries, new XYSeriesCollection(data));
	}

	public void setTitle(String title) {
		chart.setTitle(title);

		if (frame != null) {
			frame.setTitle(title);
		}
	}

	public void setXLabel(String label) {
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.getDomainAxis().setLabel(label);
	}

	public void setYLabel(String label) {
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.getRangeAxis().setLabel(label);
	}

	private SimpleFrame frame = null;

	public synchronized SimpleFrame getSimpleFrame() {
		if (frame == null) {
			frame = new SimpleFrame(chart);
		}
		return frame;
	}

	public synchronized void display() {
		SimpleFrame frame = getSimpleFrame();

		// frame.pack();
		/**
		 * Since the current version of SGT Beans doesn't directly support
		 * resizing disable resizing and make the frame visible.
		 */
		// frame.setResizable(false);
		frame.setVisible(true);

		frame.setTitle(chart.getTitle().getText());
	}

	public void saveAs(String fileName) throws java.io.IOException {
		java.io.File file = new java.io.File(fileName);

		/**
		 * Size is measured in pixels The smaller the image, the fewer tickmarks
		 * and labels from the axis are shown
		 */
		// org.jfree.chart.ChartUtilities.saveChartAsPNG(file, chart, 300, 200);
		org.jfree.chart.ChartUtilities.saveChartAsPNG(file, chart, 600, 400);
	}

	public void plot(double[] x1, double[] y1, double[] x2, double[] y2) {
		// Safety-check
		if (x1 == null || y1 == null) {
			throw new IllegalArgumentException(
					"Valid X and Y arrays should be provided");
		}
		if (x2 == null || y2 == null) {
			throw new IllegalArgumentException(
					"Valid X and Y arrays should be provided");
		}
		// Safety-check
		if (x1.length != y1.length) {
			throw new IllegalArgumentException(
					"X and Y should be of the same length");
		}
		if (x2.length != y2.length) {
			throw new IllegalArgumentException(
					"X and Y should be of the same length");
		}

		XYSeriesCollection colxn = new XYSeriesCollection();

		// XYSeries data1 = new XYSeries("Series 1"); // NEW
		// data1.isSorted(false);
		XYSeries data1 = new XYSeries("Series 1", false, true); // NEW
		int iEntries = x1.length;
		for (int i = 0; i < iEntries; i++) {
			data1.add(x1[i], y1[i]);
		}
		colxn.addSeries(data1);

		// XYSeries data2 = new XYSeries("Series 2"); // NEW
		// data2.isSorted(false);
		XYSeries data2 = new XYSeries("Series 2", false, true); // NEW
		iEntries = x2.length;
		for (int i = 0; i < iEntries; i++) {
			data2.add(x2[i], y2[i]);
		}
		colxn.addSeries(data2);

		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setDataset(colxn);
	}

	public void grid(boolean status) {
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setRangeGridlinesVisible(status);
		plot.setDomainGridlinesVisible(status);
	}

	public void axis(boolean status) {
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.getDomainAxis().setVisible(status);
		plot.getRangeAxis().setVisible(status);
	}

	public void legend(boolean status) {
		if (status) {
			// chart.setLegend(org.jfree.chart.Legend.createInstance(chart));
			chart.addLegend(new LegendTitle(chart.getPlot()));

		} else {
			chart.addLegend(null);
		}
	}

	/**
	 * Copied from DefaultDrawingSupplier Helper method to avoid lots of
	 * explicit casts in getShape(). Returns an array containing the provided
	 * doubles cast to ints.
	 * 
	 * @param a
	 *            x
	 * @param b
	 *            y
	 * @param c
	 *            z
	 * @param d
	 *            t
	 * 
	 * @return int[3] with converted params.
	 */
	private static int[] intArray(double a, double b, double c, double d) {
		return new int[] { (int) a, (int) b, (int) c, (int) d };
	}

	/**
	 * Copied from DefaultDrawingSupplier Helper method to avoid lots of
	 * explicit casts in getShape(). Returns an array containing the provided
	 * doubles cast to ints.
	 * 
	 * @param a
	 *            x
	 * @param b
	 *            y
	 * @param c
	 *            z
	 * 
	 * @return int[3] with converted params.
	 */
	private static int[] intArray(double a, double b, double c) {
		return new int[] { (int) a, (int) b, (int) c };
	}

	private static int[] intArray(double[] data) {
		int iNbrPoints = data.length;
		int[] iAryPoints = new int[iNbrPoints];

		for (int i = 0; i < iNbrPoints; i++) {
			double dData = data[i];
			int iData = (int) dData;
			iAryPoints[i] = iData;
		}

		return iAryPoints;
	}

	// /**
	// * Creates an array of standard shapes to display for the items in series
	// on charts.
	// *
	// * @return The array of shapes.
	// */
	// public static Shape[] createStandardSeriesShapes() {
	//
	// Shape[] result = new Shape[10];
	//
	// double size = 6.0;
	// double delta = size / 2.0;
	// int[] xpoints = null;
	// int[] ypoints = null;
	//
	// // square
	// result[0] = new Rectangle2D.Double(-delta, -delta, size, size);
	// // circle
	// result[1] = new Ellipse2D.Double(-delta, -delta, size, size);
	//
	// // up-pointing triangle
	// xpoints = intArray(0.0, delta, -delta);
	// ypoints = intArray(-delta, delta, delta);
	// result[2] = new Polygon(xpoints, ypoints, 3);
	//
	// // diamond
	// xpoints = intArray(0.0, delta, 0.0, -delta);
	// ypoints = intArray(-delta, 0.0, delta, 0.0);
	// result[3] = new Polygon(xpoints, ypoints, 4);
	//
	// // horizontal rectangle
	// result[4] = new Rectangle2D.Double(-delta, -delta / 2, size, size / 2);
	//
	// // down-pointing triangle
	// xpoints = intArray(-delta, +delta, 0.0);
	// ypoints = intArray(-delta, -delta, delta);
	// result[5] = new Polygon(xpoints, ypoints, 3);
	//
	// // horizontal ellipse
	// result[6] = new Ellipse2D.Double(-delta, -delta / 2, size, size / 2);
	//
	// // right-pointing triangle
	// xpoints = intArray(-delta, delta, -delta);
	// ypoints = intArray(-delta, 0.0, delta);
	// result[7] = new Polygon(xpoints, ypoints, 3);
	//
	// // vertical rectangle
	// result[8] = new Rectangle2D.Double(-delta / 2, -delta, size / 2, size);
	//
	// // left-pointing triangle
	// xpoints = intArray(-delta, delta, delta);
	// ypoints = intArray(0.0, -delta, +delta);
	// result[9] = new Polygon(xpoints, ypoints, 3);
	//
	// return result;
	//
	// }

	/**
	 * Adapted from DefaultDrawingSupplier
	 */
	public static final java.awt.Shape TRIANGLE_UP;

	public static final java.awt.Shape TRIANGLE_RIGHT;

	public static final java.awt.Shape VECTOR_HEAD;

	public static final java.awt.Shape DIAMOND;

	public static final java.awt.Shape SQUARE;

	public static final java.awt.Shape CIRCLE;

	static {
		int[] xpoints = null;
		int[] ypoints = null;

		double size = 4.0; // NORMAL
		// double size = 8.0; // BIG
		double delta = size / 2.0;

		// up-pointing triangle
		xpoints = intArray(0.0, delta * 1.5, -delta * 1.5);
		ypoints = intArray(-delta * 1.5, delta * 1.5, delta * 1.5);
		TRIANGLE_UP = new java.awt.Polygon(xpoints, ypoints, 3);

		// right-pointing triangle
		xpoints = intArray(-delta, delta, -delta);
		ypoints = intArray(-delta, 0.0, delta);
		TRIANGLE_RIGHT = new java.awt.Polygon(xpoints, ypoints, 3);

		// diamond
		xpoints = intArray(0.0, delta * 1.7, 0.0, -delta * 1.7);
		ypoints = intArray(-delta * 1.7, 0.0, delta * 1.7, 0.0);
		DIAMOND = new java.awt.Polygon(xpoints, ypoints, 4);

		// right-pointing triangle
		double dBIG = delta * 1.5;
		xpoints = intArray(-dBIG, dBIG, -dBIG);
		ypoints = intArray(-dBIG, 0.0, dBIG);
		VECTOR_HEAD = new java.awt.Polygon(xpoints, ypoints, 3);

		// SQUARE
		xpoints = intArray(-delta, +delta, +delta, -delta);
		ypoints = intArray(-delta, -delta, +delta, +delta);
		SQUARE = new java.awt.Polygon(xpoints, ypoints, 4);

		// circle
		int iNbrPoints = 9;
		xpoints = new int[iNbrPoints];
		ypoints = new int[iNbrPoints];
		for (int i = 0; i < iNbrPoints; i++) {
			double dAngle = Math.PI * 2d / iNbrPoints * i;
			xpoints[i] = (int) (delta * 1.3 * Math.cos(dAngle));
			ypoints[i] = (int) (delta * 1.3 * Math.sin(dAngle));
		}
		CIRCLE = new java.awt.Polygon(xpoints, ypoints, iNbrPoints);
	}

	private static class XY {
		public double x;

		public double y;

		public XY() {
		}

		public XY(double x, double y) {
			this.x = x;
			this.y = y;
		}
	}

	private static XY rotate(XY data, double angle) {
		angle = -angle;
		double dX = data.x * Math.cos(angle) - data.y * Math.sin(angle);
		double dY = data.x * Math.sin(angle) + data.y * Math.cos(angle);
		return new XY(dX, dY);
	}

	// public static java.awt.Shape createShape(java.awt.Shape shape, double
	// angle) {
	// }

	public static java.awt.Shape createShape(double[] aryX, double[] aryY,
			double angle) {
		// Safety-check
		if (aryX == null || aryY == null) {
			throw new IllegalArgumentException(
					"Valid arrays of data must be provided");
		}
		int iNbrPoints = aryX.length;
		if (iNbrPoints != aryY.length) {
			throw new IllegalArgumentException(
					"Both arrays must be of the same length");
		}

		double[] dAryX = new double[iNbrPoints];
		double[] dAryY = new double[iNbrPoints];

		for (int i = 0; i < iNbrPoints; i++) {
			XY data = new XY(aryX[i], aryY[i]);
			XY dataRotated = rotate(data, angle);

			dAryX[i] = dataRotated.x;
			dAryY[i] = dataRotated.y;
		}

		int[] xpoints = intArray(dAryX);
		int[] ypoints = intArray(dAryY);

		java.awt.Shape shape = new java.awt.Polygon(xpoints, ypoints,
				iNbrPoints);
		return shape;
	}

	protected static JFreeChart createLineShapePlot(String title,
			String xAxisLabel, String yAxisLabel,
			org.jfree.data.xy.XYDataset data,
			org.jfree.chart.plot.PlotOrientation orientation, boolean legend,
			boolean tooltips, boolean urls) {

		NumberAxis xAxis = new NumberAxis(xAxisLabel);
		xAxis.setAutoRangeIncludesZero(false);
		NumberAxis yAxis = new NumberAxis(yAxisLabel);
		yAxis.setAutoRangeIncludesZero(false);

		// XYPlot plot = new XYPlot(data, xAxis, yAxis, null);
		XYPlot plot = new XYPlot(data, xAxis, yAxis, null);

		org.jfree.chart.labels.XYToolTipGenerator toolTipGenerator = null;
		// org.jfree.chart.labels.XYItemLabelGenerator toolTipGenerator = null;
		if (tooltips) {
			toolTipGenerator = new org.jfree.chart.labels.StandardXYToolTipGenerator();
		}

		org.jfree.chart.urls.XYURLGenerator urlGenerator = null;
		if (urls) {
			urlGenerator = new org.jfree.chart.urls.StandardXYURLGenerator();
		}
		StandardXYItemRenderer renderer = new StandardXYItemRenderer(
				StandardXYItemRenderer.SHAPES_AND_LINES, toolTipGenerator,
				urlGenerator);
		renderer.setShapesFilled(Boolean.TRUE);
		plot.setRenderer(renderer);
		plot.setOrientation(orientation);

		JFreeChart chart = new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT,
				plot, legend);

		return chart;
	}

	public void clearPlot() {
		XYPlot plot = chart.getXYPlot();
		XYSeriesCollection colxn = (XYSeriesCollection) plot.getDataset();
		if (colxn != null) {
			colxn.removeAllSeries();
		}

		// int iSecondaryDataSets = plot.getSecondaryDatasetCount();
		// for (int i = 0; i < iSecondaryDataSets; i++) {
		// colxn = (XYSeriesCollection) plot.getSecondaryDataset(i);
		// colxn.removeAllSeries();
		// }

	}

	// /**
	// * Possible bug if the data ... ?
	// */
	// public void setEqualAxes(boolean equalAxes) {
	// XYPlot plot = chart.getXYPlot();
	// plot.useEqualAxes(equalAxes);
	// }
}
