package org.jfree.chart.renderer.xy;

import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;

import org.jfree.chart.LegendItem;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.labels.XYToolTipGenerator;
import org.jfree.chart.plot.CrosshairState;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYZDataset;
import org.jfree.ui.RectangleEdge;
import org.jfree.util.PublicCloneable;

import outils.Constantes;

/**
 * Renderer pour les graphes en "bubble"
 * part du principe que les valeurs pour les points sont entre 0 et 1
 * 
 *  @author julien.durand@gmail.com
 *
 */
public class XYBubbleRendererDynGraph extends AbstractXYItemRenderer

implements XYItemRenderer, 
Cloneable,
PublicCloneable,
Serializable {

	/** For serialization. */
	public static final long serialVersionUID = -5221991598674249125L;

	/**
	 * Constructs a new renderer.
	 */
	public XYBubbleRendererDynGraph() {
		super(); 
	}



	/**
	 * Draws the visual representation of a single data item.
	 *
	 * @param g2  the graphics device.
	 * @param state  the renderer state.
	 * @param dataArea  the area within which the data is being drawn.
	 * @param info  collects information about the drawing.
	 * @param plot  the plot (can be used to obtain standard color 
	 *              information etc).
	 * @param domainAxis  the domain (horizontal) axis.
	 * @param rangeAxis  the range (vertical) axis.
	 * @param dataset  the dataset (an {@link XYZDataset} is expected).
	 * @param series  the series index (zero-based).
	 * @param item  the item index (zero-based).
	 * @param crosshairState  crosshair information for the plot 
	 *                        (<code>null</code> permitted).
	 * @param pass  the pass index.
	 */
	public void drawItem(Graphics2D g2, XYItemRendererState state,
			Rectangle2D dataArea, PlotRenderingInfo info, XYPlot plot,
			ValueAxis domainAxis, ValueAxis rangeAxis, XYDataset dataset, 
			int series, int item, CrosshairState crosshairState, int pass) {

		PlotOrientation orientation = plot.getOrientation();

//		get the data point...
		double x = dataset.getXValue(series, item);
		double y = dataset.getYValue(series, item);
		double z = Double.NaN;
		if (dataset instanceof XYZDataset) {
			XYZDataset xyzData = (XYZDataset) dataset;
			z = xyzData.getZValue(series, item);
		}
		if (!Double.isNaN(z)) {
			RectangleEdge domainAxisLocation = plot.getDomainAxisEdge();
			RectangleEdge rangeAxisLocation = plot.getRangeAxisEdge();
			double transX = domainAxis.valueToJava2D(x, dataArea, 
					domainAxisLocation);
			double transY = rangeAxis.valueToJava2D(y, dataArea, 
					rangeAxisLocation);

			double trans = 0.0;
			
			//en supposant que 0 < z =< 1 
			//on fixe la taille du cercle à un certain pourcentage de la taille du graphe
			//la "taille" du graphe est la moyenne de sa largeur et de sa hauteur
			trans=z*Constantes.TAILLE_MAX_CERCLE_EN_POURCENTAGE*(dataArea.getHeight()+dataArea.getWidth())/2;
			trans = Math.abs(trans);
			Ellipse2D circle = null;
			if (orientation == PlotOrientation.VERTICAL) {
				circle = new Ellipse2D.Double(transX - trans / 2.0, 
						transY - trans / 2.0, trans, trans);
			}
			else if (orientation == PlotOrientation.HORIZONTAL) {
				circle = new Ellipse2D.Double(transY - trans / 2.0, 
						transX - trans / 2.0, trans, trans);
			}
			g2.setPaint(getItemPaint(series, item));
			g2.fill(circle);
			g2.setStroke(getItemOutlineStroke(series, item));
			g2.setPaint(getItemOutlinePaint(series, item));
			g2.draw(circle);

			if (isItemLabelVisible(series, item)) {
				if (orientation == PlotOrientation.VERTICAL) {
					drawItemLabel(g2, orientation, dataset, series, item, 
							transX, transY, false);
				}
				else if (orientation == PlotOrientation.HORIZONTAL) {
					drawItemLabel(g2, orientation, dataset, series, item, 
							transY, transX, false);                
				}
			}

//			setup for collecting optional entity info...
			EntityCollection entities = null;
			if (info != null) {
				entities = info.getOwner().getEntityCollection();
			}

//			add an entity for the item...
			if (entities != null) {
				String tip = null;
				XYToolTipGenerator generator 
				= getToolTipGenerator(series, item);
				if (generator != null) {
					tip = generator.generateToolTip(dataset, series, item);
				}
				String url = null;
				if (getURLGenerator() != null) {
					url = getURLGenerator().generateURL(dataset, series, item);
				}
				XYItemEntity entity = new XYItemEntity(circle, dataset, series,
						item, tip, url);
				entities.add(entity);
			}

			int domainAxisIndex = plot.getDomainAxisIndex(domainAxis);
			int rangeAxisIndex = plot.getRangeAxisIndex(rangeAxis);
			updateCrosshairValues(crosshairState, x, y, domainAxisIndex, 
					rangeAxisIndex, transX, transY, orientation);
		}

	}

	/**
	 * Returns a legend item for the specified series.  The default method
	 * is overridden so that the legend displays circles for all series.
	 *
	 * @param datasetIndex  the dataset index (zero-based).
	 * @param series  the series index (zero-based).
	 *
	 * @return A legend item for the series.
	 */
	public LegendItem getLegendItem(int datasetIndex, int series) {
		LegendItem result = null;
		XYPlot xyplot = getPlot();
		if (xyplot != null) {
			XYDataset dataset = xyplot.getDataset(datasetIndex);
			if (dataset != null) {
				if (getItemVisible(series, 0)) {
					String label = getLegendItemLabelGenerator().generateLabel(
							dataset, series);
					String description = label;
					String toolTipText = null;
					if (getLegendItemToolTipGenerator() != null) {
						toolTipText 
						= getLegendItemToolTipGenerator().generateLabel(
								dataset, series);
					}
					String urlText = null;
					if (getLegendItemURLGenerator() != null) {
						urlText = getLegendItemURLGenerator().generateLabel(
								dataset, series);
					}
					Shape shape = new Ellipse2D.Double(-4.0, -4.0, 8.0, 8.0);
					Paint paint = getSeriesPaint(series);
					Paint outlinePaint = getSeriesOutlinePaint(series);
					Stroke outlineStroke = getSeriesOutlineStroke(series);
					result = new LegendItem(label, description, 
							toolTipText, urlText, shape, paint, 
							outlineStroke, outlinePaint);
				}
			}

		}
		return result;
	}

	/**
	 * Tests this renderer for equality with an arbitrary object.
	 * 
	 * @param obj  the object (<code>null</code> permitted).
	 * 
	 * @return A boolean.
	 */
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (!(obj instanceof XYBubbleRendererDynGraph)) {
			return false;
		}
		return super.equals(obj);
	}

	/**
	 * Returns a clone of the renderer.
	 * 
	 * @return A clone.
	 * 
	 * @throws CloneNotSupportedException  if the renderer cannot be cloned.
	 */
	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

}
