/* ===========================================================
 * JFreeChart : a free chart library for the Java(tm) platform
 * ===========================================================
 *
 * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
 *
 * Project Info:  http://www.jfree.org/jfreechart/index.html
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by 
 * the Free Software Foundation; either version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA.  
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
 * in the United States and other countries.]
 *
 * ---------------------------
 * VectorSeriesCollection.java
 * ---------------------------
 * (C) Copyright 2007, by Object Refinery Limited.
 *
 * Original Author:  David Gilbert (for Object Refinery Limited);
 * Contributor(s):   -;
 *
 * $Id: VectorSeriesCollection.java,v 1.2 2008/02/16 14:47:25 julien-durand Exp $
 *
 * Changes
 * -------
 * 30-Jan-2007 : Version 1 (DG);
 *
 */

package org.jfree.experimental.data.xy;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

import org.jfree.data.KeyedValue;
import org.jfree.data.UnknownKeyException;
import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.xy.AbstractXYDataset;
import org.jfree.util.ObjectUtilities;

/**
 * A collection of {@link VectorSeries} objects.
 */
public class VectorSeriesCollection extends AbstractXYDataset implements
		VectorXYDataset, Serializable {
	private static final long serialVersionUID = 7823224514793680345L;

	/** Storage for the data series. */
	private List<VectorSeries> data;

	/**
	 * Creates a new instance of <code>VectorSeriesCollection</code>.
	 */
	public VectorSeriesCollection() {
		this.data = new java.util.ArrayList<VectorSeries>();
	}

	/**
	 * Adds a series to the collection and sends a {@link DatasetChangeEvent} to
	 * all registered listeners.
	 * 
	 * @param series
	 *            the series (<code>null</code> not permitted).
	 */
	public void addSeries(VectorSeries series) {
		if (series == null) {
			throw new IllegalArgumentException("Null 'series' argument.");
		}
		this.data.add(series);
		series.addChangeListener(this);
		fireDatasetChanged();
	}

	/**
	 * Returns the number of series in the collection.
	 * 
	 * @return The series count.
	 */
	public int getSeriesCount() {
		return this.data.size();
	}

	/**
	 * Returns a series from the collection.
	 * 
	 * @param series
	 *            the series index (zero-based).
	 * 
	 * @return The series.
	 * 
	 * @throws IllegalArgumentException
	 *             if <code>series</code> is not in the range <code>0</code>
	 *             to <code>getSeriesCount() - 1</code>.
	 */
	public VectorSeries getSeries(int series) {
		if ((series < 0) || (series >= getSeriesCount())) {
			throw new IllegalArgumentException("Series index out of bounds");
		}
		return (VectorSeries) this.data.get(series);
	}

	/**
	 * Returns the key for a series.
	 * 
	 * @param series
	 *            the series index (in the range <code>0</code> to
	 *            <code>getSeriesCount() - 1</code>).
	 * 
	 * @return The key for a series.
	 * 
	 * @throws IllegalArgumentException
	 *             if <code>series</code> is not in the specified range.
	 */
	public Comparable getSeriesKey(int series) {
		// defer argument checking
		return getSeries(series).getKey();
	}

	/**
	 * Returns the number of items in the specified series.
	 * 
	 * @param series
	 *            the series (zero-based index).
	 * 
	 * @return The item count.
	 * 
	 * @throws IllegalArgumentException
	 *             if <code>series</code> is not in the range <code>0</code>
	 *             to <code>getSeriesCount() - 1</code>.
	 */
	public int getItemCount(int series) {
		// defer argument checking
		return getSeries(series).getItemCount();
	}

	/**
	 * Returns the x-value for an item within a series.
	 * 
	 * @param series
	 *            the series index.
	 * @param item
	 *            the item index.
	 * 
	 * @return The x-value.
	 */
	public double getXValue(int series, int item) {
		VectorSeries s = (VectorSeries) this.data.get(series);
		VectorDataItem di = (VectorDataItem) s.getDataItem(item);
		return di.getXValue();
	}

	public Number getX(int series, int item) {
		return new Double(getXValue(series, item));
	}

	/**
	 * Returns the y-value for an item within a series.
	 * 
	 * @param series
	 *            the series index.
	 * @param item
	 *            the item index.
	 * 
	 * @return The y-value.
	 */
	public double getYValue(int series, int item) {
		VectorSeries s = (VectorSeries) this.data.get(series);
		VectorDataItem di = (VectorDataItem) s.getDataItem(item);
		return di.getYValue();
	}

	public Number getY(int series, int item) {
		return new Double(getYValue(series, item));
	}

	public Number getDeltaX(int series, int item) {
		return new Double(getDeltaXValue(series, item));
	}

	public double getDeltaXValue(int series, int item) {
		VectorSeries s = (VectorSeries) this.data.get(series);
		VectorDataItem di = (VectorDataItem) s.getDataItem(item);
		return di.getDeltaX();
	}

	/**
	 * Returns the end x-value for an item within a series.
	 * 
	 * @param series
	 *            the series index.
	 * @param item
	 *            the item index.
	 * 
	 * @return The x-value.
	 */
	public Number getDeltaY(int series, int item) {
		return new Double(getDeltaYValue(series, item));
	}

	public double getDeltaYValue(int series, int item) {
		VectorSeries s = (VectorSeries) this.data.get(series);
		VectorDataItem di = (VectorDataItem) s.getDataItem(item);
		return di.getDeltaY();
	}

	/**
	 * Tests this instance for equality with an arbitrary object.
	 * 
	 * @param obj
	 *            the object (<code>null</code> permitted).
	 * 
	 * @return A boolean.
	 */
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (!(obj instanceof VectorSeriesCollection)) {
			return false;
		}
		VectorSeriesCollection that = (VectorSeriesCollection) obj;
		return ObjectUtilities.equal(this.data, that.data);
	}

	/**
	 * Returns a clone of this instance.
	 * 
	 * @return A clone.
	 * 
	 * @throws CloneNotSupportedException
	 *             if there is a problem.
	 */
	public Object clone() throws CloneNotSupportedException {
		VectorSeriesCollection clone = (VectorSeriesCollection) super.clone();
		clone.data = this.data;
		return clone;
	}

	/**
	 * Returns the index for a given key.
	 * 
	 * @param key
	 *            the key (<code>null</code> not permitted).
	 * 
	 * @return The index, or <code>-1</code> if the key is not recognised.
	 * 
	 * @throws IllegalArgumentException
	 *             if <code>key</code> is <code>null</code>.
	 */
	public int getIndex(Comparable key) {
		if (key == null) {
			throw new IllegalArgumentException("Null 'key' argument.");
		}
		int i = 0;
		Iterator iterator = this.data.iterator();
		while (iterator.hasNext()) {
			KeyedValue kv = (KeyedValue) iterator.next();
			if (kv.getKey().equals(key)) {
				return i;
			}
			i++;
		}
		return -1; // key not found
	}

	/**
	 * Returns a key.
	 * 
	 * @param index
	 *            the item index (zero-based).
	 * 
	 * @return The row key.
	 * 
	 * @throws IndexOutOfBoundsException
	 *             if <code>item</code> is out of bounds.
	 */
	public Comparable getKey(int index) {
		Comparable result = null;
		KeyedValue item = (KeyedValue) this.data.get(index);
		if (item != null) {
			result = item.getKey();
		}
		return result;
	}

	/**
	 * Returns the keys for the values in the collection.
	 * 
	 * @return The keys (never <code>null</code>).
	 */
	public List<VectorSeries> getKeys() {
		/*
		 * List result = new java.util.ArrayList(); Iterator iterator =
		 * this.data.iterator(); while (iterator.hasNext()) { KeyedValue kv =
		 * (KeyedValue) iterator.next(); result.add(kv.getKey()); } return
		 * result;
		 */
		return this.data;
	}

	/**
	 * Returns the value for a given key.
	 * 
	 * @param key
	 *            the key.
	 * 
	 * @return The value (possibly <code>null</code>).
	 * 
	 * @throws UnknownKeyException
	 *             if the key is not recognised.
	 */
	public Number getValue(Comparable key) {
		int index = getIndex(key);
		if (index < 0) {
			throw new UnknownKeyException("Key not found: " + key);
		}
		return getValue(index);
	}

	/**
	 * Returns the number of items (values) in the collection.
	 * 
	 * @return The item count.
	 */
	public int getItemCount() {
		return this.data.size();
	}

	/**
	 * Returns a value.
	 * 
	 * @param item
	 *            the item of interest (zero-based index).
	 * 
	 * @return The value.
	 * 
	 * @throws IndexOutOfBoundsException
	 *             if <code>item</code> is out of bounds.
	 */
	public Number getValue(int item) {
		Number result = null;
		KeyedValue kval = (KeyedValue) this.data.get(item);
		if (kval != null) {
			result = kval.getValue();
		}
		return result;
	}

}
